addHook("NetVars", function(x)
	WARE = x($)
end)

rawset(_G, "WARE_RESET", function()
	// Zeroes some variables
	WARE.boss = false
	WARE.round = 0
	for p in players.iterate do
		p.wareperm = {score=0}
	end
end)
rawset(_G, "WARE_CLEANUP", function()
	// Restores WARE to clean slate
	WARE = {
		prepped = false,
		state = WS_WAITPLAYERS,
		time = 0,
		round = 0,
		boss = false,
		speed = 0,
		deluxe = false,
		lethality = 0,
	}
end)

rawset(_G, "WARE_PlayJingle", function(str, p)
	--S_ChangeMusic("WJBOS"..WARE.speed, false)
	str = $..min(WARE.speed, 4)
	if WARE.deluxe then str = "DELUXE" end
	S_ChangeMusic(str..WARE.speed, false, p)
end)
rawset(_G, "WARE_PlayMusic", function(str, loop)
	if WARE.deluxe then str = "DELUXE" end
	S_ChangeMusic(str, loop or false)
end)

rawset(_G, "ISWAREGAMETYPE", function()
	return gametype >= GT_WARE and gametype <= GT_WAREDEBUG
end)
rawset(_G, "ISWARE", function()
	return ISWAREGAMETYPE() and (mapheaderinfo[gamemap].typeoflevel & TOL_WARE) > 0
end)
local ixes = { // More like degrees of freedom
	[GT_WARE] = 1, -- The original, attempts to prevent cheating or irregular behaviour
	[GT_WARECUSTOM] = 2, -- The spinoff - A bit lax on irregularities, and lets the host change some settings
	[GT_WAREDEBUG] = 3, -- The bootleg - Does not check for weird behaviour, and you can use debug commands
}
rawset(_G, "WHATWARE", function()
	-- tbh fucking useless
	return ixes[gametype]
end)

rawset(_G, "WAREPLY_INIT", function(p)
	// player ware memory to blank style gamer
	p.waremem = {
		won = false,
		wonlock = false,
	}
	p.wareperm = {
		inplay = false,
		init = true,
		score = 0,
	}
end)
rawset(_G, "WAREPLY_CLEANUP", function()
	// Restores player WARE memory to clean slate
	for p in players.iterate do
		p.waremem = {
			won = false,
			wonlock = false,
		}
	end
end)
rawset(_G, "WAREPLY_PLAYNOW", function()
	// Sets player as "in play"
	for p in players.iterate do
		p.wareperm.inplay = true
	end
	
end)

rawset(_G, "WARE_PlayerInField", function(p)
	// Is this player in the field?
	// i.e. not spectator
	return p.mo and p.mo.valid and not p.spectator
end)
rawset(_G, "WARE_PlayerInPlay", function(p)
	// Is this player *playing*?
	// i.e. not spectator when the minigame started
	return WARE_PlayerInField(p) and p.wareperm.inplay
end)
rawset(_G, "WARE_PlayerCompeting", function(p)
	// Is this player competing?
	// That is, they haven't won or lost yet, and they are playing
	return WARE_PlayerInPlay(p) and not p.waremem.wonlock
end)
rawset(_G, "WARE_GetPlaying", function()
	local plys = {}
	for p in players.iterate do
		if WARE_PlayerInPlay(p) and p.playerstate ~= PST_DEAD then plys[#plys+1] = p end
	end
	return plys
end)
rawset(_G, "WARE_ForEachPlayer", function(callback)
	local plys = {}
	for p in players.iterate do
		--if WARE_PlayerInPlay(p) then plys[#plys+1] = p end
		if WARE_PlayerInPlay(p) then
			callback(p)
		end
	end
	--for i = 1,#plys do callback(plys[i],i) end
end)
rawset(_G, "WARE_ForEachCompetingPlayer", function(callback)
	local plys = {}
	for p in players.iterate do
		if WARE_PlayerCompeting(p) then
			callback(p)
		end
	end
end)

rawset(_G, "WARE_PlayerSetStatus", function(p, winning, lock)
	// Set player winning status
	if p.waremem and not p.waremem.wonlock then
		p.waremem.won = winning
		if lock then
			p.waremem.wonlock = true
			if winning then
				for p2 in players.iterate do
					local me = p == p2
					S_StartSound(p.mo,me and sfx_wawin or sfx_wawin2, p2)
				end
			else
				S_StartSound(p.mo,sfx_lose)
			end
		end
		return true
	else return false end
end)
rawset(_G, "WARE_PlayerWin", function(p, lock)
	// This player won the minigame!
	WARE_PlayerSetStatus(p, true, lock)
end)
rawset(_G, "WARE_PlayerLose", function(p, lock)
	// This player lost the minigame...
	// The player loses by default if they haven't won yet
	WARE_PlayerSetStatus(p, false, lock)
end)

rawset(_G, "WARE_RestorePlayers", function()
	local plys = WARE_GetPlaying()
	for i = 1,#plys do
		local p = plys[i]
		local m = p.mo
		P_ResetPlayer(p)
		m.state = S_PLAY_STND
		m.momx,m.momy,m.momz = 0,0,0
		m.scale = FRACUNIT
		p.charability = skins[m.skin].ability
		p.charability2 = skins[m.skin].ability2
	end
end)
rawset(_G, "WARE_WarpPlayers", function(where, pos)
	// Move everyone
	WARE_RestorePlayers()
	local plys = shuffleTable(WARE_GetPlaying()) // for cool placement
	local kind = pos and pos[1] or nil
	
	if kind then
		if kind == "radial" then
			local sec = 360*FRACUNIT / #plys
			local off = P_RandomRange(0,359)*FRACUNIT
			for i = 1,#plys do
				local p = plys[i]
				
				local xr,yr = cSpd(FixedAngle(off+sec*i), pos[2] or 0)
				P_TeleportMove(p.mo,where.x*FRACUNIT+xr,where.y*FRACUNIT+yr,where.z*FRACUNIT)
			end
		elseif kind == "scatter" then
			// takes two corners
			// {"scatter",{x=-3840,y=256,z=0},{x=-2304,y=-128,z=0}}
			local c1 = pos[2]
			local c2 = pos[3]
			for i = 1,#plys do
				P_TeleportMove(plys[i].mo,
					P_RandomRange(c1.x, c2.x)*FRACUNIT,
					P_RandomRange(c1.y, c2.y)*FRACUNIT,
					P_RandomRange(c1.z, c2.z)*FRACUNIT
				)
			end
		end
	else
		for i = 1,#plys do
			local p = plys[i]
			P_TeleportMove(p.mo,where.x*FRACUNIT,where.y*FRACUNIT,where.z*FRACUNIT)
		end
	end
end)
rawset(_G, "WARE_BackToOrigin", function()
	// Put everyone back to origin
	//WARE_WarpPlayers({x=0,y=0,z=0},{"scatter"})
	WARE_WarpPlayers({x=0,y=0,z=0})
end)

rawset(_G, "WARE_ForcePlayerSkin", function(p, skin, color)
	// Force this player to wear this skin (and color)
	local wm = p.waremem
	R_SetPlayerSkin(p, skin)
	wm.forceskin = skin
	if color ~= nil then
		wm.forcecolor = color
	end
end)


local platforms = {7,6,10,9,5,4,3,8,1} // 16, 32, 48 hardcoded af
rawset(_G, "WARE_FixOrigin", function()
	// Restores the starting area lmao
	local lava = sectors[19]
	lava.ceilingheight = -TICRATE*FRACUNIT
	for i = 1,#platforms do
		local ix = platforms[i]
		local j = i+3
		sectors[ix].floorheight = (j/4)*16*FRACUNIT
	end
end)


rawset(_G, "MODULWARE", true)